import pandas as pd
import numpy as np

from pymarket import Market
from pymarket.transactions import TransactionManager, split_transactions_merged_players
from pymarket.bids.processing import merge_same_price
from pymarket.mechanisms.muda_auction import solve_market_side_with_exogenous_price


def test_split_transactions():
    """
    Test the reconstruction
    of the transaction list
    of a merged bids dataframe
    """

    mar = Market()

    mar.bm.add_bid(1, 100, 0, True, 0)
    mar.bm.add_bid(3, 100, 1, True, 0)
    mar.bm.add_bid(2.3, 85, 2, True, 0)
    mar.bm.add_bid(2.1, 90, 7, True, 0)
    mar.bm.add_bid(0.4, 90, 8, True, 0)

    mar.bm.add_bid(0.5, 90, 4, False, 0)
    mar.bm.add_bid(4.2, 1, 5, False, 0)
    mar.bm.add_bid(0.1, 90, 6, False, 0)

    df = mar.bm.get_df()
    df_, maping = merge_same_price(df)

    trans, fee = solve_market_side_with_exogenous_price(df_, 95, np.arange(df_.shape[0] + 1))

    new_trans = split_transactions_merged_players(trans, df, maping)

    X_obtained = new_trans.get_df().sort_values('bid').values.astype('float')

    X_true = np.array([
        [0, 1, 95, -1, False],
        [1, 3, 95, -1, False],
        [6, 4, 95, -1, False]
    ]).astype(float)

    assert np.allclose(X_obtained, X_true)



import pytest
from pymarket.bids import BidManager


@pytest.fixture(scope='module')
def bid_dataset_0():
    """
    Simple bid dataset in which demand intersects supply
    """

    bm = BidManager()
    bm.add_bid(1, 3, 0, True, 0)
    bm.add_bid(2, 4, 1, True, 0)
    bm.add_bid(5, 1, 2, True, 0)

    bm.add_bid(4, 2, 3, False, 0)
    bm.add_bid(1, 1, 4, False, 0)
    bm.add_bid(5, 6, 5, False, 0)
        
    return bm.get_df()


@pytest.fixture(scope='module')
def bidlist_dataset_0():
    """
    Simple bid dataset made of bid of lists.
    Same as `bid_dataset_0` in which demand intersects supply
    """

    bm = BidManager()
    bm.add_bid(1, 3, 0, True, 0)
    bm.add_bid(2, 4, 1, True, 0)
    bm.add_bid(5, 1, 2, True, 0)

    bm.add_bid(4, 2, 3, False, 0)
    bm.add_bid(1, 1, 4, False, 0)
    bm.add_bid(5, 6, 5, False, 0)
    
    return bm.get_df()

@pytest.fixture(scope='module')
def bid_dataset_1():
    """
    Larger test case, supply intersects demand
    """

    bm = BidManager()

    bm.add_bid(1, 6.7, 0, True, 0)
    bm.add_bid(1, 6.6, 1, True, 0)
    bm.add_bid(1, 6.5, 2, True, 0)
    bm.add_bid(1, 6.4, 3, True, 0)
    bm.add_bid(1, 6.3, 4, True, 0)
    bm.add_bid(1, 6, 5, True, 0)

    bm.add_bid(1, 1, 6, False, 0)
    bm.add_bid(1, 2, 7, False, 0)
    bm.add_bid(2, 3, 8, False, 0)
    bm.add_bid(2, 4, 9, False, 0)
    bm.add_bid(1, 6.1, 10, False, 0)
    
    return bm.get_df()

@pytest.fixture(scope='module')
def bid_dataset_muda_example_1():
    """Simple example with 3 biders as described
    in the MUDA paper
    Returns
    -------
    df : pandas.DataFrame
        dataframe with all the bids

    """
    bm = BidManager()
    bm.add_bid(1, 100, 0, True, 0)
    bm.add_bid(1, 90, 0, True, 0)
    bm.add_bid(1, 80, 0, True, 0)
    bm.add_bid(1, 60, 0, True, 0)
    bm.add_bid(1, 40, 0, True, 0)
    bm.add_bid(1, 20, 0, True, 0)

    bm.add_bid(1, 10, 1, False, 0)
    bm.add_bid(1, 20, 1, False, 0)
    bm.add_bid(1, 40, 1, False, 0)
    bm.add_bid(1, 60, 1, False, 0)
    bm.add_bid(1, 70, 1, False, 0)
    
    bm.add_bid(1, 15, 2, False, 0)
    bm.add_bid(1, 25, 2, False, 0)
    bm.add_bid(1, 35, 2, False, 0)
    bm.add_bid(1, 45, 2, False, 0)
    bm.add_bid(1, 65, 2, False, 0)
    
    return bm.get_df()


@pytest.fixture(scope='module')
def bid_dataset_3():
    
    bm = BidManager()
    bm.add_bid(1   , 6 , 0 , True , 0)
    bm.add_bid(0.1 , 5 , 1 , True , 0)
    bm.add_bid(1.9 , 4 , 2 , True , 0)
    bm.add_bid(1   , 3 , 3 , True , 0)

    bm.add_bid(1   , 1   , 4 , False , 0)
    bm.add_bid(1.5 , 2   , 5 , False , 0)
    bm.add_bid(1   , 2.5 , 6 , False , 0)
    bm.add_bid(1   , 4   , 7 , False , 0)

    return bm.get_df()



import numpy as np
from pymarket.bids import *
from pymarket.transactions import *

from pymarket.mechanisms import p2p_random


def test_p2p_random_dataset_0(bid_dataset_0):
    """ Tests the p2p-random mechanisms in the dataset
    0 with several possible orders

    Order of trading:

    Seed = 1234
    Round 0 :
        1-3 : q=2, p=3
        2-4 : q=1, p=1
        0-5 : q=0, p=0
    Round 1 :
        2-3 : q=0, p=0
    Round 2 :
        0-3 : q=1, p=2.5
        2-5 : q=0, p=0 

    Parameters
    ----------
    bid_dataset_0 : dataframe created from the bid_0 dataset

    Returns
    -------
    

    """
    df = bid_dataset_0
    r = np.random.RandomState(1234)
    trans, extra = p2p_random(df, r=r)

    trading_list = [
            [(1 , 3)  , (2 , 4)  , (0 , 5)] ,
            [(2 , 3)] ,
            [(0 , 3)  , (2 , 5)]
            ]

    assert trading_list == extra['trading_list']

    true_trans = [[1, 2, 3.0, 3, False],
             [3, 2, 3.0, 1, True],
            [2, 1, 1.0, 4, True],
             [4, 1, 1.0, 2, False],
             [0, 0, 0.0, 5, True],
             [5, 0, 0.0, 0, True],
             [2, 0, 0.0, 3, True],
             [3, 0, 0.0, 2, True],
             [0, 1, 2.5, 3, False],
             [3, 1, 2.5, 0, True],
             [2, 0, 0.0, 5, True],
             [5, 0, 0.0, 2, True],]

    true_trans = np.array(true_trans).astype(float)
    
    df = trans.get_df().values.astype(float)
    assert np.allclose(df, true_trans)




import pytest
import pandas as pd
import numpy as np
from pymarket.bids import BidManager
from pymarket.bids.processing import merge_same_price


def test_bids(bid_dataset_0):
    """
    Check that the bid_dataset_0
    is exactly as expected
    """
    df = bid_dataset_0
    
    data = [
      [ 1 , 3 , 0 , True  , 0 , True],
      [ 2 , 4 , 1 , True  , 0 , True],
      [ 5 , 1 , 2 , True  , 0 , True],
      [ 4 , 2 , 3 , False , 0 , True],
      [ 1 , 1 , 4 , False , 0 , True],
      [ 5 , 6 , 5 , False , 0 , True],
            ]
    columns = ['quantity', 'price', 'user', 'buying', 'time', 'divisible']
    df2 = pd.DataFrame(data, columns=columns)

    assert df2.equals(df)


def test_merge_same_price():
    """    Test the merge_same_price
    functionalty with repeated
    price in both sides and equal
    price for the repated price.
    """

    bm = BidManager()

    bm.add_bid(1, 100, 0, True, 0)
    bm.add_bid(3, 100, 1, True, 0)
    bm.add_bid(2.3, 85, 2, True, 0)
    bm.add_bid(2.1, 90, 7, True, 0)
    bm.add_bid(0.4, 90, 8, True, 0)

    bm.add_bid(0.5, 90, 4, False, 0)
    bm.add_bid(4.2, 1, 5, False, 0)
    bm.add_bid(0.1, 90, 6, False, 0)
    df = bm.get_df()

    df_new, maping = merge_same_price(df)

    X_original = np.array([
        [2.3, 85, 2, True, 0, True],
        [4.2, 1, 5, False, 0, True],
        [2.5, 90, 9, True, 0, True],
        [4, 100, 10, True, 0, True],
        [0.6, 90, 11, False, 0, True],
    ]).astype(float)
    
    maping_original = {
        0: [2], 1: [3, 4], 2: [0, 1], 3: [6], 4: [5, 7]
    }

    X_new = df_new.sort_values('user').values.astype(float)

    assert np.allclose(X_original, X_new)
    for k in maping_original:
        assert maping_original[k] == maping[k]